// $Id: JavaMethodInfo.h,v 1.10 1998/07/30 21:08:28 nickm Exp $
// Copyright (c) 1995  David Engberg  All rights reserved
#ifndef _JavaMethodInfo_h
#define _JavaMethodInfo_h
#pragma interface

#include "JavaClassFile.h"
#include "JavaMethodSignature.h"
#include "JavaAccessFlags.h"
#include "PrimitiveMethods.h"
class CJavaCodeAttribute;
class CJavaExceptionsTable;
class ostream;
class CCompiler;

//
//  Class name : CJavaMethodInfo
//  Description : This class encapsulates the information found in one method
//    on a Java class.  This includes indices for the name and signature of
//    the method as well as access flags and code for the method.
//
class CJavaMethodInfo {
public:
  static CJavaMethodInfo* ParseBuffer(string::const_iterator& javaBuffer,
				      const CJavaClassFile& classFile,
				      bool interfaceOnly = false);

  const CJavaMethodSignature& GetSignature() const { return fSignature; }
  const CJavaAccessFlags& GetModifiers() const { return fAccessFlags; }
  const CJavaCodeAttribute* GetCode() const { return fCodeAttribute; }
  const CJavaMethodSignature& GetTranslatedSignature() const;

  CJavaCodeAttribute* GetCode() { return fCodeAttribute; }

  void Disassemble(ostream& toStream) const;

  void AddReference() const;
  void RemoveReference() const;
  bool IsDeprecated() const;

  bool ThrowsExceptions() const;
  deque<unicode_string>::const_iterator ThrowsBegin() const;
  deque<unicode_string>::const_iterator ThrowsEnd() const;
protected:
  CJavaMethodInfo();
  ~CJavaMethodInfo();
private:
  friend class CCompiler;
  friend class CPJavaClassInfo;  
  friend CJavaClassFile* CreateClassForType(const CJavaTypeSignature&, 
					    const unicode_string&);

  void SetSignature(const CJavaMethodSignature& sig) const;
  void SetTranslatedSignature(const CJavaMethodSignature& sig) const;

  const deque<CJavaTypeSignature> &GetThrows() const;

  CJavaAccessFlags fAccessFlags;
  CJavaMethodSignature fSignature;
  CJavaMethodSignature fTranslatedSignature;
  CJavaCodeAttribute* fCodeAttribute;
  CJavaExceptionsTable* fExceptions;
  unsigned long fReferenceCount;
  bool fDeprecated;
  deque<CJavaTypeSignature> *fThrows;
};

#endif
