// find_leaks.c
//
// If FIND_LEAKS is defined, implements a reference-counter to selected 
// pointers.

#ifdef FIND_LEAKS
#include <iostream.h>
#include <map>
#include "find_leaks.h"
#include "instantiation_macros.h"

typedef const void *Pointer;

struct ptr_info {
  Pointer ptr;
  const char *fname;
  unsigned long line;
  bool ignore;
};

typedef map<Pointer, ptr_info *, less<Pointer> > pMap;

DECL_MAP(Pointer, ptr_info *);
DECL_LESS(Pointer);

pMap gPointerMap;

void 
register_pointer(Pointer p, const char *file, unsigned long line)
{
  pMap::iterator i = gPointerMap.find(p);
  if (i != gPointerMap.end()) {
    if (!(*i).second->ignore) {
      cerr << "*** Warning: Pointer " << p << " registered twice: " << endl
	   << "1st at line " << (*i).second->line << " of " 
	   << (*i).second->fname << endl
	   << "2st at line " << line << " of " << file << endl;
    }
  } else {
    ptr_info *info = new ptr_info;
    info->ptr = p;
    info->fname = file;
    info->line = line;
    info->ignore = false;
    gPointerMap.insert(pMap::value_type(p,info));
  }
}

void 
unregister_pointer(Pointer p)
{
  pMap::iterator i = gPointerMap.find(p);
  if (i != gPointerMap.end()) {
    if (! (*i).second->ignore) {
      gPointerMap.erase(i);
      delete (*i).second;
    }
  } else {
    cerr << "*** Warning: Pointer " << p 
	 << " unregistered, but not registered first." << endl;
  }
}

void 
ignore_pointer(Pointer p)
{
  pMap::iterator i = gPointerMap.find(p);
  if (i != gPointerMap.end()) {
    (*i).second->ignore = true;
  } else {
    ptr_info *info = new ptr_info;
    info->ptr = p;
    info->fname = NULL;
    info->line = 0;
    info->ignore = true;
    gPointerMap.insert(pMap::value_type(p,info));
  }
}

void memory_trace()
{
  for (pMap::iterator i = gPointerMap.begin(); 
       i != gPointerMap.end(); ++i) {
    if (! (*i).second->ignore) {
      cerr << "Not freed: " << (*i).second->ptr << " (from line " 
	   << (*i).second->line << " of " << (*i).second->fname << ")" << endl;
    }
  }
}

#endif
