// Copyright (c) 1996  David Engberg  All rights reserved
// $Id: IntermediateClass.C,v 1.13 1998/08/07 15:59:26 nickm Exp $
#pragma implementation
#include "find_leaks.h"
#include "IntermediateClass.h"
#include "IntermediateFunction.h"
#include "Statement.h"
#include "PJavaClassInfo.h"
#include <iostream.h>

class CSourceFile;

//
//  Method name : CIntermediateClass
//  Description : Constructs a class.
//
CIntermediateClass::CIntermediateClass(CSourceFile *sourceFile,
				       const string& sourceFileName,
				       const unicode_string& className,
				       CJavaAccessFlags* adoptModifiers,
				       CJavaTypeSignature* adoptExtends,
				    deque<CJavaTypeSignature>* adoptInterfaces,
				       bool deprecated,
				       unsigned long startLineNumber)
  : fSourceFile(sourceFile),
    fSourceFileName(sourceFileName),
    fName(className),
    fExtends(0),
    fStaticLocalVariables(0),
    fStaticInitializer(0),
    fRealClass(0),
    fDeprecated(deprecated),
    fStartLineNumber(startLineNumber),
    fPClassInfo(0),
    fType(0)
{
  if (adoptModifiers != 0) {
    fAccessFlags = *adoptModifiers;
    delete adoptModifiers;
  }
  if (adoptExtends != 0) {
    assert(adoptExtends->IsReference());
    assert(! adoptExtends->IsArray());
    fExtends = new unicode_string();
    adoptExtends->GetBaseClassName(*fExtends);
  }
  if (adoptInterfaces != 0) {
    for(deque<CJavaTypeSignature>::iterator i = adoptInterfaces->begin();
        i != adoptInterfaces->end(); ++i) {
      assert((*i).IsReference());
      assert(! (*i).IsArray());
      unicode_string *className = new unicode_string();      
      (*i).GetBaseClassName(*className);
      fInterfaces.push_back(*className);
    }
  }
}

//
//  Method name : ~CIntermediateClass
//  Description : Destructor
//
CIntermediateClass::~CIntermediateClass()
{
  delete fExtends;
  for (deque<CIntermediateFunction*>::iterator i = fFunctions.begin();
       i != fFunctions.end(); i++) {
    delete *i;
  }  
  delete fStaticInitializer;
  for (StatementList::iterator i = fStaticDeclarations.begin();
       !(i == fStaticDeclarations.end()); ++i) {
    delete *i;
  }
  for (StatementList::iterator i = fNonStaticDeclarations.begin();
       !(i == fNonStaticDeclarations.end()); ++i) {
    delete *i;
  }
  delete fRealClass;
  delete fType;
}

//
//  Method name : GetShortName
//  Description : This method is used to get the part of the class name after
//    the last package delimiter.  For example, if this class name is
//    java.foo.Blah, then this will return "Blah".  If this class name is
//    just "Blah", then it will return "Blah".
//
unicode_string
CIntermediateClass::GetShortName() const
{
  unicode_string shortName = fName;
  unicode_string::size_type lastSlash = fName.find_last_of('/');
  if (lastSlash != unicode_string::npos) {
    shortName.assign(fName, lastSlash + 1, unicode_string::npos);
  }  
  return shortName;
}

//
//  Method name : SetType
//  Description : Private method used to set fType to this class's type.
//
void
CIntermediateClass::SetType()
{
  if (fType)
    return;
  if (! fPClassInfo || ! fPClassInfo->IsParameterized()) {
    fType = new CJavaTypeSignature(fName);
  } else {
    deque<CJavaTypeSignature>* params = new deque<CJavaTypeSignature>;
    for (deque<unicode_string>::const_iterator p = fPClassInfo->ParamsBegin();
	 p != fPClassInfo->ParamsEnd(); ++p) {
      CJavaTypeSignature t(*p, 0, 0, true);
      params->push_back(t);
    }
    fType = new CJavaTypeSignature(fName,0,params);
  }
}

string CIntermediateClass::Disassemble() const {
    return SymbolToUTF(fName);
}

ostream &operator<<(ostream &o, CIntermediateClass const &c) {
    o << c.Disassemble();
    return o;
}
