// Copyright (c) 1995  David Engberg  All rights reserved
// $Id: VariableDeclaration.h,v 1.9 1998/07/28 20:38:33 nickm Exp $
#ifndef _VariableDeclaration_h
#define _VariableDeclaration_h
#pragma interface

#include "unicode_string.h"
#include "JavaFieldSignature.h"
class CExpression;
class CCompileError;
class CJavaMethodInfo;
class CJavaClassFile;
class CCompiler;
class CJavaAccessFlags;
class CCompileContext;
class srcstream;

//
//  Class name : CVariableDeclaration
//  Description : This class is used as the intermediate representation of
//    a field or variable declaration in Java source, including the
//    initializer.  This value is either a straightforward expression or
//    a CArrayInitializer, which allows shorthand initialization of arrays.
//
class CVariableDeclaration {
public:
  CVariableDeclaration(const CJavaTypeSignature& type,
		       const unicode_string& name,
		       CExpression* adoptInitializer = 0);
  ~CVariableDeclaration();

  CJavaTypeSignature GetType() const;
  void SetType(const CJavaTypeSignature& type);
  const CJavaFieldSignature& GetSignature() const { return fSignature; }
  CCompileError* GetConstantValue(CExpression*& intoPointer,
				  CCompileContext& context);
  CCompileError* Unparse(srcstream &sOut,
				 CCompileContext &context);
  unsigned long GetLineNumber() const { return fLineNumber; }
  bool HasInitializer() const { return (fInitializer != 0); }
  CCompileError* UnparseInitializer(srcstream &sOut,
					    CCompileContext &context);
  CCompileError* UnparseNoInitializer(srcstream &sOut,
					      CCompileContext &context);
private:
  CJavaFieldSignature fSignature;
  CExpression* fInitializer;
  unsigned long fLineNumber;
};

#endif
